(*
   3DPLOT V1.0

   3D function plotter
   Bernd Bock
*)
program Plot3D;
(*$IKERNEL2.INC*)
(*$IINPUT.INC*)

const
  MaxSteps      =  60;
  ScreenWidth   = 720;
  ScreenHeight  = 256;
  ScreenFactorX =  46;
  ScreenFactorY = 120;
label
  byebye;
var
  selection: char;
  selectedFunc: char;
  kx, gx, ky, gy: integer;
  sx: integer;
  m: integer;
  g: real;


(* Displays the menu and returns the selected item. *)
function menu: char;
var ch: char;
begin
  ClrScr;
  GotoXY(38,  2); Write('*** 3DPLOT ***');
  GotoXY(40,  3); Write('Bernd Bock');
  GotoXY(31,  8); Write('Funktion ausw{hlen........1');
  GotoXY(31,  9); Write('Funktion plotten..........2');
  GotoXY(31, 10); Write('Programmende..............3');
  
  ch := '-';
  while (ch < '1') or (ch > '3') do
    ch := getKey;
  
  menu := ch;
end; (* menu *)


(* Selects the function calculate. *)
procedure selectFunction;
var ch: char;
begin
  ClrScr;
  GotoXY(32,  2); Write('*** Funktion ausw{hlen ***');
  GotoXY(31,  6); Write('z = sin(sqr(x*x + y*y))...1');
  GotoXY(31,  7); Write('z = sin(x*x + y*y)........2');
  
  ch := '-';
  while (ch < '1') or (ch > '2') do
    ch := getKey;
  
  selectedFunc := ch;
end; (* selectFunction *)


(* Inputs the range, number of steps, z stretch, and zoom factor. *)
procedure inputParameters;
var ch: char;
begin
  ClrScr;
  ShowCursor;
  Write('x-min: '); ReadLn(kx); WriteLn;
  Write('x-max: '); ReadLn(gx); WriteLn;
  Write('y-min: '); ReadLn(ky); WriteLn;
  Write('y-max: '); ReadLn(gy); WriteLn;
  Write('Anzahl Schritte (10-60): '); ReadLn(sx); WriteLn;
  Write('z-Streckung (1000-20000): '); ReadLn(m); WriteLn;
  Write('Vergr|~erung (1=Normal): '); ReadLn(g); WriteLn;
end; (* inputParameters *)


(* Calculates the selected 3D function. *)
function calc(x, y: real): real;
var z: real;
begin
  case selectedFunc of
    '1': z := sin(sqrt(x*x + y*y));
    '2': z := sin(x*x + y*y);
  end;
  
  calc := z;
end; (* calc *)


(* Draws a line; converts from GSX to screen coordinates. *)
procedure drawLine(x1, y1, x2, y2: integer);
begin
  x1 := x1 div ScreenFactorX;
  y1 := y1 div ScreenFactorY;
  x2 := x2 div ScreenFactorX;
  y2 := y2 div ScreenFactorY;
  x1 := ScreenWidth - x1;
  x2 := ScreenWidth - x2;
  Line(x1, y1, x2, y2, 0);
end; (* plotLine *)


(* Plots the selected function. *)
procedure plotFunction;
var sy: real;
    x: array(.0..MaxSteps,0..MaxSteps.) of integer;
    y: array(.0..MaxSteps,0..MaxSteps.) of integer;
    dx, dy, s: real;
    isy: integer;
    i, j: integer;
    u, v: real;
    z: real;
    x1, y1, x2, y2: integer;
begin
  inputParameters;
  HideCursor;

  sy := 0.7 * sx;
  dx := (gx - kx) / sx;
  dy := (gy - ky) / sy;
  s := 15000 / sx * g;

  WriteLn; WriteLn('Funktionswerte werden berechnet');

  isy := trunc(sy + 0.001);

  for i := sx downto 0 do begin
    for j := isy downto 0 do begin
      u := kx + i * dx;
      v := ky + j * dy;
      z := calc(u, v);
      z := m * z;
      x(.i, j.) := 32722 - trunc(41000.0 / (g + 2) + (i - j) * s * 1.2);
      y(.i, j.) := trunc(8786 / g + j * s - z);
    end;
  end;

  WriteLn; WriteLn('Verdeckte Fl{chen werden berechnet');

  for i := sx downto 1 do begin
    for j := isy downto 1 do begin
      if y(.pred(i), pred(j).) > y(.i, j.) then y(.pred(i), pred(j).) := y(.i, j.);
    end;
  end;
  
  ClrScr;
  
  for i := 0 to pred(sx) do begin
    for j := 0 to pred(isy) do begin
      (* Berechnung der Grafik *)
      if i * dx < 32722 then begin
        x1 := x(.i, j.);
        y1 := y(.i, j.);
        x2 := x(.succ(i), j.);
        y2 := y(.succ(i), j.);
        drawLine(x1, y1, x2, y2);
      end;
      if j * dy < 32636 then begin
        x1 := x(.i, j.);
        y1 := y(.i, j.);
        x2 := x(.i, succ(j).);
        y2 := y(.i, succ(j).);
        drawLine(x1, y1, x2, y2);
      end;
    end;

    x1 := x(.i, isy.);
    y1 := y(.i, isy.);
    x2 := x(.succ(i), isy.);
    y2 := y(.succ(i), isy.);
    drawLine(x1, y1, x2, y2);
  end;

  for j := 0 to pred(isy) do begin
    x2 := x(.sx, j.);
    y2 := y(.sx, j.);
    x1 := x(.sx, succ(j).);
    y1 := y(.sx, succ(j).);
    drawLine(x1, y1, x2, y2);
  end;

  Beep;
  WaitForKey;
end; (* plotFunction *)


(* Displays a finish message. *)
procedure finish;
begin
  WriteLn;
  WriteLn;
  WriteLn;
  WriteLn('Ende');
end; (* finish *)


(* Main program *)
begin
  Write(chr(27), '0'); (* switch off status bar *)
  GraphInit;
  selectedFunc := '1'; (* default function *)
  
  repeat
      HideCursor;
      selection := menu;
      case selection of
        '1': selectFunction;
        '2': plotFunction;
        '3': finish;
      end;
  until selection = '3';
  
byebye:
  Write(chr(27), '1'); (* switch on status bar *)
  ShowCursor;
end.
